@@ -1,4 +1,7 @@
 package Signals::XSIG;
+
+## no critic (RequireLocalizedPunctuationVars, ProhibitAutomaticExport)
+
 use Signals::XSIG::Default;
 use Carp;
 use Config;
@@ -12,7 +15,7 @@ our @ISA = qw(Exporter);
 our @EXPORT = qw(%XSIG);
 our @EXPORT_OK = qw(untied);
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 our (%XSIG, %_XSIG, %SIGTABLE, $_REFRESH, $_DISABLE_WARNINGS);
 our $_INITIALIZED = 0;
 our $SIGTIE = bless {}, 'Signals::XSIG::TieSIG';
@@ -66,14 +69,26 @@ sub _init {
   }
   $SIGTABLE{'__WARN__'} = '__WARN__';
   $SIGTABLE{'__DIE__'} = '__DIE__';
-  $_INITIALIZED++;
+  return ++$_INITIALIZED;
 }
 
-sub __shadow__warn__handler { &__shadow_signal_handler('__WARN__',@_) }
-sub __shadow__die__handler  { &__shadow_signal_handler('__DIE__',@_) }
+sub __shadow__warn__handler {          ## no critic (Unpacking)
+  return &__shadow_signal_handler('__WARN__',@_) 
+}
+sub __shadow__die__handler  {          ## no critic (Unpacking)
+  return &__shadow_signal_handler('__DIE__',@_) 
+}
+
+our $_GLOBAL_DESTRUCTION = 0;
+END {
+  $_GLOBAL_DESTRUCTION = 1; 
+}
 
 sub __shadow_signal_handler {
   my ($signal, @args) = @_;
+
+  # %XSIG might be partially or completely untied during global destruction
+  return if $_GLOBAL_DESTRUCTION;
   my $seen_default = 0;
 
   my $h = tied @{$XSIG{$signal}};
@@ -100,9 +115,9 @@ sub __shadow_signal_handler {
 	}
       }
       next if $seen_default++;
-      Signals::XSIG::Default::perform_default_behavior(@_);
+      Signals::XSIG::Default::perform_default_behavior($signal, @args);
     } else {
-      no strict 'refs';
+      no strict 'refs';                    ## no critic (NoStrict)
       if ($signal =~ /__\w+__/) {
 	$subhandler->(@args);
       } else {
@@ -110,6 +125,7 @@ sub __shadow_signal_handler {
       }
     }
   }
+  return;
 }
 
 # convert a signal name to its canonical name. If not disabled,
@@ -122,24 +138,25 @@ sub _resolve_signal {
   $DISABLE_WARNINGS ||= $_DISABLE_WARNINGS;
   $sig = $SIGTABLE{uc $sig};
   if (defined $sig) {
-    $_[0] = $sig;
+    $_[0] = $sig;  ## no critic (Unpacking)
     return 1;
   }
   return 1 if !$_INITIALIZED;
 
   # signal could not be resolved -- issue warning and return false
   unless ($DISABLE_WARNINGS) {
-    if ($_[0] =~ /\d/ && $_[0] !~ /\D/) {
-      carp "Invalid signal number $_[0].\n";
+    if (defined($sig) && $sig =~ /\d/ && $sig !~ /\D/) {
+      carp "Invalid signal number $sig.\n";
     } elsif (warnings::enabled('signal')) {
-      Carp::cluck "Invalid signal name $_[0].\n";
+      Carp::cluck "Invalid signal name $sig.\n";
     }
   }
   return;
 }
 
 # execute a block of code while %SIG is temporarily untied.
-sub untied (&) {
+
+sub untied (&) {                    ## no critic (SubroutinePrototypes)
   my $BLOCK = shift;
 
   untie %SIG;
@@ -149,12 +166,14 @@ sub untied (&) {
   return wantarray ? @r : $r[0];
 }
 
+
+
 # in %SIG and %XSIG assignments, string values are qualified to the
 # 'main' package, unqualified *glob values are qualified to the
 # calling package.
 sub _qualify_handler {
   my $handler = shift;
-  if (defined $handler && $handler ne ''
+  if (defined($handler) && $handler ne ''
       && $handler ne 'IGNORE' && $handler ne 'DEFAULT') {
 
     my $qr = qr/^Signals::XSIG/;
@@ -203,7 +222,7 @@ sub Signals::XSIG::TieSIG::STORE {
   } else {
     my $old;
     untied {
-      no warnings 'signal';
+      no warnings 'signal';          ## no critic (NoWarnings)
       $old = $SIG{$key};
       $SIG{$key} = $value;
     };
@@ -227,6 +246,7 @@ sub Signals::XSIG::TieSIG::DELETE {
 sub Signals::XSIG::TieSIG::CLEAR {
   my ($self) = @_;
   $_XSIG{$_}[0] = undef for keys %XSIG;
+  return;
 }
 
 sub Signals::XSIG::TieSIG::EXISTS {
@@ -237,7 +257,7 @@ sub Signals::XSIG::TieSIG::EXISTS {
 sub Signals::XSIG::TieSIG::FIRSTKEY {
   my ($self) = @_;
   my $a = keys %_XSIG;
-  each %_XSIG;
+  return each %_XSIG;
 }
 
 sub Signals::XSIG::TieSIG::NEXTKEY {
@@ -323,9 +343,10 @@ sub Signals::XSIG::TieArray::_refresh_SIG {
     }
   }
   untied {
-    no warnings qw(uninitialized signal);
+    no warnings qw(uninitialized signal); ## no critic (NoWarnings)
     $SIG{$sig} = $handler_to_install;
   };
+  return;
 }
 
 sub Signals::XSIG::TieArray::handlers {
@@ -371,6 +392,7 @@ sub Signals::XSIG::TieArray::UNSHIFT {
   unshift @{$self->{handlers}}, @list;
   $self->{start} -= @list;
   $self->_refresh_SIG;
+  return $self->FETCHSIZE;
 }
 
 sub Signals::XSIG::TieArray::POP {
@@ -522,6 +544,7 @@ sub Signals::XSIG::TieXSIG::CLEAR {
     my ($xsig, $alias) = @$pair;
     $_XSIG{$alias} = $_XSIG{$xsig};
   }
+  return;
 }
 
 sub Signals::XSIG::TieXSIG::EXISTS {
@@ -532,7 +555,7 @@ sub Signals::XSIG::TieXSIG::EXISTS {
 sub Signals::XSIG::TieXSIG::FIRSTKEY {
   my ($self) = @_;
   my $a = keys %_XSIG;
-  each %_XSIG;
+  return each %_XSIG;
 }
 
 sub Signals::XSIG::TieXSIG::NEXTKEY {
@@ -550,7 +573,7 @@ Signals::XSIG - install multiple signal handlers through %XSIG
 
 =head1 VERSION
 
-Version 0.10
+Version 0.11
 
 =head1 SYNOPSIS
 
@@ -578,7 +601,8 @@ Version 0.10
 =head1 DESCRIPTION
 
 Perl provides the magic global hash variable C<%SIG> to make it
-easy to trap and handle signals (see L<perlvar/"%SIG"> and L<perlipc>).
+easy to trap and handle signals (see L<perlvar/"%SIG"> and 
+L<perlipc|perlipc>).
 The hash-of-lists variable C<%XSIG> provided by this module
 has a similar interface for setting an arbitrary number of
 handlers on any signal.
@@ -809,7 +833,8 @@ Marty O'Brien, C<< <mob at cpan.org> >>
 =item Avoid C<local %SIG>
 
 This module converts C<%SIG> into a tied hash. As documented in 
-L<perltie|/"BUGS">, C<local>izing a tied hash will cause the old data
+L<the perltie "BUGS" section|perltie/"BUGS">,
+C<local>izing a tied hash will cause the old data
 not to be restored when the local version of the hash goes out of scope.
 Avoid doing this:
 
@@ -819,9 +844,9 @@ Avoid doing this:
     }
 
 or using modules and functions which localize C<%SIG> 
-(fortunately, there are
-L<not that many|http://www.google.com/codesearch?hl=en&lr=&q=%22local+%25SIG%22+lang:perl&sbtn=Search> 
-examples of code that use this construction).
+(fortunately, there are not that many examples of code that
+use this construction 
+[L<http://www.google.com/codesearch?hl=en&lr=&q=%22local+%25SIG%22+lang:perl&sbtn=Search>]).
 
 Should you identify a code block that localizes C<%SIG> and you can't/don't
 want to avoid using it, the workaround is to save and restore C<%SIG> at
@@ -833,6 +858,11 @@ the end of the local scope:
     function_call_or_block_that_localizes_SIG();
     %SIG = %temp;
 
+In addition, the behavior of the tied C<%SIG> while it is C<local>'ized
+is different in different versions of Perl, and all of the features
+of C<Signals::XSIG> might or might not work while a local copy
+of C<%SIG> is in use.
+
 Note that it is perfectly fine to C<local>ize an I<element> of C<%SIG>:
 
     {
@@ -919,3 +949,5 @@ by the Free Software Foundation; or the Artistic License.
 See http://dev.perl.org/licenses/ for more information.
 
 =cut
+
+
